#!/usr/bin/env python
# coding=utf8
__author__ = 'ldath'

from flask import url_for
from flask_wtf import Form
from flask_lwadmin import ConfigurationError


class ConfigParser:
    NO_URL = 0
    URL_INTERNAL = 1
    URL_EXTERNAL = 2
    URL_CALL = 3
    HTML = 4

    def __init__(self):
        self.list_configuration = dict(
            id={},
            display=[],
            actions=[],
            object_actions=[],
            batch={},
            filter={}
        )

    def configure(self, configuration):
        if 'list' in configuration.keys():
            if 'display' in configuration['list'].keys():
                self.parse_list_display(configuration['list']['display'])

            if 'actions' in configuration['list'].keys():
                self.parse_list_actions(configuration['list']['actions'])

            if 'object_actions' in configuration['list'].keys():
                self.parse_list_object_actions(configuration['list']['object_actions'])

            if 'batch' in configuration['list'].keys():
                self.parse_batch(configuration['list']['batch'])

            if 'filter' in configuration['list'].keys():
                self.parse_filter(configuration['list']['filter'])

    def parse_list_display(self, elements):
        for element in elements:
            if not all(k in element for k in ('key', 'label')):
                raise ConfigurationError('Wrong configuration format for list display element')
            self.list_configuration['display'].append(element)

    def parse_list_actions(self, actions):
        for action in actions:
            parsed = self.parse_action(action)
            self.list_configuration['actions'].append(parsed)

    def parse_list_object_actions(self, actions):
        for action in actions:
            parsed = self.parse_action(action)
            self.list_configuration['object_actions'].append(parsed)

    def parse_action(self, action):
        if not all(k in action for k in ('key', 'label')):
            raise ConfigurationError('Wrong configuration format for list actions element')

        if 'type' not in action.keys():
            action['type'] = self.NO_URL

        if 'credential' not in action.keys():
            action['credential'] = None

        if 'confirm' not in action.keys():
            action['confirm'] = False

        if 'confirm_message' not in action.keys():
            action['confirm_message'] = 'Are you sure?'

        if 'class' not in action.keys():
            action['class'] = 'btn btn-small'

        if 'call' not in action.keys():
            action['call'] = False

        if 'visable' not in action.keys():
            action['visable'] = True

        if 'disabled' not in action.keys():
            action['disabled'] = False

        return action

    def parse_batch(self, batch_elemet):
        if not all(k in batch_elemet for k in ('url', 'form')):
            raise ConfigurationError('Wrong configuration format for list filter element')

        if 'type' not in batch_elemet.keys():
            batch_elemet['type'] = self.NO_URL

        self.list_configuration['batch'] = batch_elemet

    def parse_filter(self, filter_elemet):
        if not all(k in filter_elemet for k in ('url', 'form')):
            raise ConfigurationError('Wrong configuration format for list filter element')

        if 'type' not in filter_elemet.keys():
            filter_elemet['type'] = self.NO_URL

        self.list_configuration['filter'] = filter_elemet

    def is_list_actions(self):
        return True if self.list_configuration.get('actions', []) else False

    def get_pk(self):
        return self.list_configuration.get('pk')

    def get_list_display(self):
        return self.list_configuration.get('display', [])

    def get_list_actions(self):
        actions = self.list_configuration.get('actions', [])
        for action in actions:
            pre = action.copy()
            if pre['type'] == self.URL_INTERNAL:
                pre['url'] = url_for(pre['url'])

            yield pre

    def is_list_object_actions(self):
        return True if self.list_configuration.get('object_actions', []) else False

    def get_list_object_actions(self, call_object=None):
        actions = self.list_configuration.get('object_actions', [])
        for action in actions:
            pre = action.copy()
            if pre['type'] == self.URL_INTERNAL:
                pre['url'] = url_for(pre['url'])

            if pre['key'] == 'delete':
                pre['form'] = Form()

            if pre['call'] and call_object is not None:
                pre['type'] = self.URL_CALL
                pre = getattr(call_object, pre['call'])(pre)

            yield pre

    def is_batch(self):
        return True if self.list_configuration.get('batch', {}) else False

    def get_batch(self):
        batch_elemet = self.list_configuration.get('batch', {})

        pre = batch_elemet.copy()
        if pre['type'] == self.URL_INTERNAL:
            pre['url'] = url_for(pre['url'])
        return pre

    def is_filter(self):
        return True if self.list_configuration.get('filter', {}) else False

    def get_filter(self):
        filter_elemet = self.list_configuration.get('filter', {})

        pre = filter_elemet.copy()
        if pre['type'] == self.URL_INTERNAL:
            pre['url'] = url_for(pre['url'])
        return pre



#!/usr/bin/env python
# coding=utf8
from flask import Blueprint


class ConfigurationError(RuntimeError):
    pass


class LwAdmin(object):

    def __init__(self, app=None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        blueprint = Blueprint(
            'lw_admin',
            __name__,
            template_folder='templates',
            static_folder='static',
            static_url_path=app.static_url_path + '/lw_admin')

        app.register_blueprint(blueprint)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import Flask, render_template
from flask.ext.jqueryuibootstrap import JqueryUiBootstrap
from flask.ext.lwadmin import LwAdmin
from flask.ext.lwadmin.navbar import Navbar, create_navbar_fd

app = Flask(__name__)
JqueryUiBootstrap(app)

# LwAdmin
LwAdmin(app)

navbar_conf = {
    'brand': {'brand_name': 'Test Project', 'brand_url': '/'},
    'items': [
        {
            'key': 'app.homepage',
            'label': 'Homepage',
            'url': 'homepage',
            'type': Navbar.URL_INTERNAL
        },
        {
            'key': 'app.about',
            'label': 'About',
            'url': 'about',
            'type': Navbar.URL_INTERNAL
        },
        {
            'key': 'app.contact',
            'label': 'Contact',
            'url': 'contact',
            'type': Navbar.URL_INTERNAL
        }
    ],
    'profile': [
        {
            'key': 'lw_user',
            'label': 'anonymous',
            'icon': 'icon-user'
        },
        {
            'key': 'lw_logout',
            'label': 'logout',
            'icon': 'icon-signout',
            'only_icon': True
        }
    ]
}


@app.route('/')
def homepage():
    navbar = create_navbar_fd(navbar_conf, 'app.homepage')
    return render_template('homepage.html', lw_navbar=navbar)


@app.route('/about')
def about():
    navbar = create_navbar_fd(navbar_conf, 'app.about')
    return render_template('about.html', lw_navbar=navbar)


@app.route('/contact')
def contact():
    navbar = create_navbar_fd(navbar_conf)
    navbar.set_active('app.contact')
    return render_template('contact.html', lw_navbar=navbar)


if __name__ == '__main__':
    app.run(debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import


def test_assert():
    a = 1
    b = 1
    assert a == b



#!/usr/bin/env python
# coding=utf8
from flask_lwadmin import ConfigurationError
from flask import url_for


class Navbar(object):
    NO_URL = 0
    URL_INTERNAL = 1
    URL_EXTERNAL = 2
    DIVIDER = 3
    GROUP = 4

    def __init__(self):
        self._navbar = {'brand': {'brand_name': None, 'brand_html': None, 'brand_url': None}, 'items': [], 'profile': []}
        self._items = {}
        self._keys = []
        self._permissions = {}
        self.menu = []
        self.profile = []

    def set_permissions(self, permissions):
        self._permissions = permissions

    def set_brand(self, brand_name=None, brand_url=None, brand_html=None):
        self._navbar['brand']['brand_name'] = brand_name
        self._navbar['brand']['brand_url'] = brand_url
        self._navbar['brand']['brand_html'] = brand_html

    def set_active(self, key):
        item = self.get_item(key)
        item['active'] = True
        self._items[key] = item

    def set_icon(self, key, icon, only_icon=False):
        item = self.get_item(key)
        item['icon'] = icon
        item['only_icon'] = only_icon
        self._items[key] = item

    def set_caret(self, key, caret):
        item = self.get_item(key)
        item['caret'] = caret
        self._items[key] = item

    def add_menu_item(self, key, label, url=None, item_type=None, credential=None, disabled=False):
        self.__create_base_item(key, label, url, item_type, credential, disabled)
        self._items[key]['dropdown'] = []
        self._navbar['items'].append(key)

    def add_profile_item(self, key, label, url=None, item_type=None, credential=None, disabled=False):
        self.__create_base_item(key, label, url, item_type, credential, disabled)
        self._items[key]['dropdown'] = []
        self._navbar['profile'].append(key)

    def add_group_item(self, parent_key, key, label, url=None, item_type=None, credential=None, disabled=False):
        self.__create_base_item(key, label, url, item_type, credential, disabled)
        self._items[key]['parent'] = parent_key
        self._items[key]['dropdown'] = []
        self._items[parent_key]['group'].append(key)

    def add_dropdown_item(self, parent_key, key, label, url=None, item_type=None, credential=None, disabled=False):
        self.__create_base_item(key, label, url, item_type, credential, disabled)
        self._items[key]['parent'] = parent_key
        self._items[parent_key]['dropdown'].append(key)

    def generate_menu(self):
        self.menu = self.__generate(self._navbar['items'])

    def generate_profile(self):
        self.profile = self.__generate(self._navbar['profile'])

    def get_item(self, key):
        if key not in self._items:
            ConfigurationError('This key: %s not exists' % key)
        return self._items[key]

    def get_data(self):
        return self._navbar

    def get_brand(self):
        return self._navbar['brand']

    def parse_item(self, item):
        if 'key' not in item.keys():
            ConfigurationError('Menu items must have unique key')

        if 'type' not in item.keys():
            item['type'] = self.NO_URL

        if 'group' in item.keys():
            item['label'] = ''
            item['type'] = self.GROUP

        if 'label' not in item.keys():
            ConfigurationError('Menu items must have label')

        item['caret'] = item.get('caret', False)

        return item

    def __create_base_item(self, key, label, url=None, item_type=None, credential=None, disabled=False):
        self.__check_key(key)
        if item_type is self.GROUP:
            self.__create_group_element(key)
        else:
            self.__create_normal_element(key, label, url, item_type, credential, disabled)

    def __create_group_element(self, key):
        item = {'type': self.GROUP, 'group': []}
        self._keys.append(key)
        self._items[key] = item

    def __create_normal_element(self, key, label, url, item_type, credential=None, disabled=False):
        if item_type == self.NO_URL:
            url = '#'

        item = {
            'label': label,
            'type': item_type,
            'url': url,
            'disabled': disabled,
            'active': False,
            'icon': None,
            'only_icon': False,
            'hidden': False,
            'caret': False
        }
        if credential is not None and self._permissions is not None:
            if credential in self._permissions.keys():
                user_permissions = self._permissions[credential]
                item['hidden'] = self.__is_hidden(user_permissions)

        self._keys.append(key)
        self._items[key] = item

    def __check_key(self, key):
        if key in self._items:
            ConfigurationError('This key: %s is not unique' % key)
        return True

    @staticmethod
    def __is_hidden(user_permission):
        if user_permission is not None:
            if not user_permission.can():
                return True
        return False

    def __generate(self, data):
        result = []
        for key in data:
            item = self.__prepare_item(self.get_item(key))
            if item['type'] == self.GROUP:
                item['group'] = self.__generate_group(item['group'])
            if len(item['dropdown']) > 0:
                item['dropdown'] = self.__generate_dropdown(item['dropdown'])

            result.append(item)
        return result

    def __generate_group(self, group):
        result = []
        for gkey in group:
            gitem = self.__prepare_item(self.get_item(gkey))
            if len(gitem['dropdown']) > 0:
                gitem['dropdown'] = self.__generate_dropdown(gitem['dropdown'])
            result.append(gitem)
        return result

    def __generate_dropdown(self, dropdowns):
        result = []
        for dropdown in dropdowns:
            item = self.__prepare_item(self.get_item(dropdown))
            result.append(item)
        return result

    def __prepare_item(self, item):
        if item['type'] == self.URL_INTERNAL:
            item['url'] = url_for(item['url'])
        return item


def create_navbar_fd(conf=None, active_key=None):
    """Creating navbar from dictionary type configuration"""

    conf = conf.copy() if conf else {}

    navbar = Navbar()
    brand = conf.get('brand', {})
    navbar.set_brand(
        brand_name=brand.get('brand_name', None),
        brand_url=brand.get('brand_url', None),
        brand_html=brand.get('brand_html', None)
    )
    if 'permissions' in conf.keys():
        navbar.set_permissions(conf['permissions'])

    items = conf.get('items', [])
    for item in items:
        add_menu_item(navbar, item)

    profile = conf.get('profile', [])
    for item in profile:
        add_profile_item(navbar, item)

    if active_key is not None:
        navbar.set_active(active_key)

    navbar.generate_menu()
    navbar.generate_profile()

    return navbar


def add_menu_item(navbar, item):
    item = navbar.parse_item(item)
    navbar.add_menu_item(
        item['key'],
        item['label'],
        item.get('url', None),
        item.get('type', None),
        item.get('credential', None),
        item.get('disabled', False)
    )

    if 'icon' in item.keys():
        navbar.set_icon(item['key'], item['icon'], item.get('only_icon', False))

    if item['caret']:
        navbar.set_caret(item['key'], True)

    if item['type'] == Navbar.GROUP:
        for subitem in item['group']:
            add_group_item(navbar, item['key'], subitem)

    if 'dropdown' in item.keys():
        for subitem in item['dropdown']:
            add_dropdown_item(navbar, item['key'], subitem)


def add_profile_item(navbar, item):
    item = navbar.parse_item(item)

    navbar.add_profile_item(
        item['key'],
        item['label'],
        item.get('url', None),
        item.get('type', None),
        item.get('credential', None),
        item.get('disabled', False)
    )

    if 'icon' in item.keys():
        navbar.set_icon(item['key'], item['icon'], item.get('only_icon', False))

    if item['caret']:
        navbar.set_caret(item['key'], True)

    if item['type'] == Navbar.GROUP:
        for subitem in item['group']:
            add_group_item(navbar, item['key'], subitem)

    if 'dropdown' in item.keys():
        for subitem in item['dropdown']:
            add_dropdown_item(navbar, item['key'], subitem)


def add_group_item(navbar, parent, item):
    item = navbar.parse_item(item)
    navbar.add_group_item(
        parent,
        item['key'],
        item['label'],
        item.get('url', None),
        item.get('type', None),
        item.get('credential', None),
        item.get('disabled', False)
    )

    if 'icon' in item.keys():
        navbar.set_icon(item['key'], item['icon'], item.get('only_icon', False))

    if item['caret']:
        navbar.set_caret(item['key'], True)

    if 'dropdown' in item.keys():
        for subitem in item['dropdown']:
            add_dropdown_item(navbar, item['key'], subitem)


def add_dropdown_item(navbar, parent, item):
    item = navbar.parse_item(item)
    navbar.add_dropdown_item(
        parent,
        item['key'],
        item['label'],
        item.get('url', None),
        item.get('type', None),
        item.get('credential', None),
        item.get('disabled', False)
    )

    if 'icon' in item.keys():
        navbar.set_icon(item['key'], item['icon'], item.get('only_icon', False))

    if item['caret']:
        navbar.set_caret(item['key'], True)



#!/usr/bin/env python
# coding=utf8
from math import floor, ceil
from flask_lwadmin.config import ConfigParser


class Pager(ConfigParser):

    def __init__(self, max_per_page=10, page=1):
        ConfigParser.__init__(self)
        self.page = int(page)
        self.max_per_page = int(max_per_page)
        self.last_page = 1
        self.nb_results = 0
        self.current_max_link = 1
        self.max_record_limit = False
        self.offset = 0
        self.limit = max_per_page

        self.results = None
        self.results_counter = 0

    def set_count(self, count):
        self.results_counter = count

    def set_results(self, results):
        self.results = results

    def initialize(self, configuration):
        has_max_record_limit = (self.get_max_record_limit() is not False)
        max_record_limit = self.get_max_record_limit()

        if has_max_record_limit:
            self.set_nb_results(min(self.results_counter, max_record_limit))
        else:
            self.set_nb_results(self.results_counter)

        page = self.get_page()
        if (0 == self.get_page()) or (0 == self.get_max_per_page()):
            self.set_last_page(0)
        else:
            self.set_last_page(int(ceil(self.get_nb_results() / float(self.get_max_per_page()))))
            self.offset = (page - 1) * self.get_max_per_page()
            if has_max_record_limit:
                max_record_limit -= self.offset
                if max_record_limit > self.get_max_per_page():
                    self.limit = self.get_max_per_page()
                else:
                    self.limit = max_record_limit
            else:
                self.limit = self.get_max_per_page()

        self.configure(configuration)

    def get_offset(self):
        """Return offset for query"""
        return self.offset

    def get_limit(self):
        """Return limit for query"""
        return self.limit

    def get_results(self):
        """Returns an list of results on the given page."""
        return self.results

    def get_max_record_limit(self):
        """Returns the current pager's max record limit."""
        return self.max_record_limit

    def set_max_record_limit(self, limit):
        """Sets the current pager's max record limit."""
        self.max_record_limit = limit

    def get_current_max_link(self):
        """Returns the current pager's max link"""
        return self.current_max_link

    def get_links(self, nb_links=5):
        """
        Returns an array of page numbers to use in pagination links.

        nb_links The maximum number of page numbers to return
        """
        page = self.page
        last_page = self.last_page
        links = []
        tmp = page - floor(nb_links / 2)
        check = last_page - nb_links + 1
        if check > 0:
            limit = check
        else:
            limit = 1

        if tmp > 0:
            if tmp > limit:
                begin = limit
            else:
                begin = tmp
        else:
            begin = 1

        i = int(begin)
        while (i < (begin + nb_links)) and (i <= last_page):
            links.append(i)
            i += 1

        if len(links):
            self.current_max_link = links[len(links) - 1]
        else:
            self.current_max_link = 1

        return links

    def have_to_paginate(self):
        """Returns true if the current query requires pagination."""
        if self.get_max_per_page() and self.get_nb_results() > self.get_max_per_page():
            return True
        return False

    def get_first_indice(self):
        """Returns the first index on the current page."""
        if self.page == 0:
            return 1
        else:
            return (self.page - 1) * self.max_per_page + 1

    def get_last_indice(self):
        """Returns the last index on the current page."""
        if self.page == 0:
            return self.nb_results
        else:
            if (self.page * self.max_per_page) >= self.nb_results:
                return self.nb_results
            else:
                return self.page * self.max_per_page

    def get_nb_results(self):
        """Returns the number of results."""
        return self.nb_results

    def set_nb_results(self, nb):
        """Sets the number of results."""
        self.nb_results = nb

    @property
    def get_first_page(self):
        """Returns the first page number."""
        return 1

    def get_last_page(self):
        """Returns the last page number."""
        return self.last_page

    def set_last_page(self, page):
        """Sets the last page number."""
        self.last_page = page
        if self.get_page() > page:
            self.set_page(page)

    def get_page(self):
        """Returns the current page."""
        return self.page

    def get_next_page(self):
        """Returns the next page."""
        return min((self.get_page() + 1), self.get_last_page())

    def get_previous_page(self):
        """Returns the previous page."""
        return max((self.get_page() - 1), self.get_first_page)

    def set_page(self, page):
        """Sets the current page."""
        self.page = int(page)

        if self.page <= 0:
            # set first page, which depends on a maximum set
            if self.get_max_per_page() > 0:
                self.page = 1
            else:
                self.page = 0

    def get_max_per_page(self):
        """Returns the maximum number of results per page."""
        return self.max_per_page

    def set_max_per_page(self, max_per_page):
        """Sets the maximum number of results per page."""
        if max_per_page > 0:
            self.max_per_page = max_per_page
            if self.page == 0:
                self.page = 1
        elif max_per_page == 0:
            self.max_per_page = 0
            self.page = 0
        else:
            self.max_per_page = 1
            if self.page == 0:
                self.page = 1

    def is_first_page(self):
        """Returns true if on the first page."""
        return 1 == self.page

    def is_last_page(self):
        """Returns true if on the last page."""
        return self.page == self.last_page

    def count(self):
        return self.nb_results



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os


def is_package(path):
    return (
        os.path.isdir(path) and
        os.path.isfile(os.path.join(path, '__init__.py'))
    )


def find_packages(path=".", base=""):
    """ Find all packages in path """
    packages = {}
    for item in os.listdir(path):
        dir = os.path.join(path, item)
        if is_package( dir ):
            if base:
                module_name = "%(base)s.%(item)s" % vars()
            else:
                module_name = item
            packages[module_name] = dir
            packages.update(find_packages(dir, module_name))
    return packages


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import with_statement, division, absolute_import


try:
    from setuptools import setup, Command, find_packages
except ImportError:
    from distutils.core import setup, Command
    from findpackages import find_packages


def get_version():
    """Get current version from VERSION file"""
    with open("VERSION") as f:
        return f.readline().strip()


def get_description():
    """Get current package description"""
    with open("DESCRIPTION") as f:
        return f.read()


def get_requirements():
    with open("requirements.txt") as f:
        return [line.strip() for line in f]


def main():
    __version__ = get_version()
    __description__ = get_description()
    __packages__ = find_packages()
    __requirements__ = get_requirements()

    setup(
        name='Flask-LwAdmin',
        version=__version__,
        author="Arkadiusz Tułodziecki",
        author_email="atulodzi@gmail.com",
        description=__description__,
        long_description=open('README.rst').read() + '\n\n' + open('HISTORY.rst').read(),
        license="MIT",
        url="https://github.com/lightningwolf/Flask-LwAdmin",
        packages=__packages__,
        zip_safe=False,
        include_package_data=True,
        platforms=['any'],
        install_requires=__requirements__,
        classifiers=[
            "Development Status :: 3 - Alpha",
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'Natural Language :: Polish',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
            'Topic :: Software Development :: Libraries :: Python Modules'
        ],
    )


if __name__ == '__main__':
    main()